syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface LogicalCameraSensor
/// \brief Information about a logical camera sensor element

message LogicalCameraSensor
{
  /// \brief Near clipping plane of the view frustum in meters.
  required double near_clip       = 1;

  /// \brief Far clipping plane of the view frustum in meters.
  required double far_clip        = 2;

  /// \brief Horizontal field of view in radians.
  required double horizontal_fov  = 3;

  /// \brief Near and far clipping plane aspect ratio (width/height).
  required double aspect_ratio    = 4;
}
