syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Scene
/// \brief A message containing a description of a scene


import "color.proto";
import "fog.proto";
import "sky.proto";
import "light.proto";
import "joint.proto";
import "model.proto";

message Scene
{
  required string name         = 1;
  optional Color ambient       = 2;
  optional Color background    = 3;
  optional Sky sky             = 4;
  optional bool shadows        = 5 [default = true];
  optional Fog fog             = 6;
  optional bool grid           = 7;

  repeated Model model         = 8;
  repeated Light light         = 9;
  repeated Joint joint         = 10;

  /// \brief Show/hide world origin indicator.
  optional bool origin_visual  = 11;
}
