// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: joint_wrench.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_joint_5fwrench_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_joint_5fwrench_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3013000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3013000 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_table_driven.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/inlined_string_field.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "wrench.pb.h"
#ifndef _MSC_VER
#pragma GCC system_header
#pragma GCC diagnostic ignored "-Wshadow"
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif
#include <boost/shared_ptr.hpp>
#include <gazebo/util/system.hh>
#include "gazebo/msgs/MsgFactory.hh"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_joint_5fwrench_2eproto GZ_MSGS_VISIBLE
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct GZ_MSGS_VISIBLE TableStruct_joint_5fwrench_2eproto {
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTableField entries[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::AuxiliaryParseTableField aux[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTable schema[1]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::FieldMetadata field_metadata[];
  static const ::PROTOBUF_NAMESPACE_ID::internal::SerializationTable serialization_table[];
  static const ::PROTOBUF_NAMESPACE_ID::uint32 offsets[];
};
extern GZ_MSGS_VISIBLE const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_joint_5fwrench_2eproto;
namespace gazebo {
namespace msgs {
class JointWrench;
class JointWrenchDefaultTypeInternal;
GZ_MSGS_VISIBLE extern JointWrenchDefaultTypeInternal _JointWrench_default_instance_;
}  // namespace msgs
}  // namespace gazebo
PROTOBUF_NAMESPACE_OPEN
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::JointWrench* Arena::CreateMaybeMessage<::gazebo::msgs::JointWrench>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace gazebo {
namespace msgs {

// ===================================================================

class GZ_MSGS_VISIBLE JointWrench PROTOBUF_FINAL :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.JointWrench) */ {
 public:
  inline JointWrench() : JointWrench(nullptr) {}
  virtual ~JointWrench();

  JointWrench(const JointWrench& from);
  JointWrench(JointWrench&& from) noexcept
    : JointWrench() {
    *this = ::std::move(from);
  }

  inline JointWrench& operator=(const JointWrench& from) {
    CopyFrom(from);
    return *this;
  }
  inline JointWrench& operator=(JointWrench&& from) noexcept {
    if (GetArena() == from.GetArena()) {
      if (this != &from) InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return GetMetadataStatic().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return GetMetadataStatic().reflection;
  }
  static const JointWrench& default_instance();

  static void InitAsDefaultInstance();  // FOR INTERNAL USE ONLY
  static inline const JointWrench* internal_default_instance() {
    return reinterpret_cast<const JointWrench*>(
               &_JointWrench_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(JointWrench& a, JointWrench& b) {
    a.Swap(&b);
  }
  inline void Swap(JointWrench* other) {
    if (other == this) return;
    if (GetArena() == other->GetArena()) {
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(JointWrench* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  inline JointWrench* New() const final {
    return CreateMaybeMessage<JointWrench>(nullptr);
  }

  JointWrench* New(::PROTOBUF_NAMESPACE_ID::Arena* arena) const final {
    return CreateMaybeMessage<JointWrench>(arena);
  }
  void CopyFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void MergeFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void CopyFrom(const JointWrench& from);
  void MergeFrom(const JointWrench& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  ::PROTOBUF_NAMESPACE_ID::uint8* _InternalSerialize(
      ::PROTOBUF_NAMESPACE_ID::uint8* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  inline void SharedCtor();
  inline void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(JointWrench* other);
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.JointWrench";
  }
  protected:
  explicit JointWrench(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  private:
  static void ArenaDtor(void* object);
  inline void RegisterArenaDtor(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  public:

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  private:
  static ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadataStatic() {
    ::PROTOBUF_NAMESPACE_ID::internal::AssignDescriptors(&::descriptor_table_joint_5fwrench_2eproto);
    return ::descriptor_table_joint_5fwrench_2eproto.file_level_metadata[kIndexInFileMessages];
  }

  public:

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kBody1NameFieldNumber = 1,
    kBody2NameFieldNumber = 3,
    kBody1WrenchFieldNumber = 5,
    kBody2WrenchFieldNumber = 6,
    kBody1IdFieldNumber = 2,
    kBody2IdFieldNumber = 4,
  };
  // required string body_1_name = 1;
  bool has_body_1_name() const;
  private:
  bool _internal_has_body_1_name() const;
  public:
  void clear_body_1_name();
  const std::string& body_1_name() const;
  void set_body_1_name(const std::string& value);
  void set_body_1_name(std::string&& value);
  void set_body_1_name(const char* value);
  void set_body_1_name(const char* value, size_t size);
  std::string* mutable_body_1_name();
  std::string* release_body_1_name();
  void set_allocated_body_1_name(std::string* body_1_name);
  private:
  const std::string& _internal_body_1_name() const;
  void _internal_set_body_1_name(const std::string& value);
  std::string* _internal_mutable_body_1_name();
  public:

  // required string body_2_name = 3;
  bool has_body_2_name() const;
  private:
  bool _internal_has_body_2_name() const;
  public:
  void clear_body_2_name();
  const std::string& body_2_name() const;
  void set_body_2_name(const std::string& value);
  void set_body_2_name(std::string&& value);
  void set_body_2_name(const char* value);
  void set_body_2_name(const char* value, size_t size);
  std::string* mutable_body_2_name();
  std::string* release_body_2_name();
  void set_allocated_body_2_name(std::string* body_2_name);
  private:
  const std::string& _internal_body_2_name() const;
  void _internal_set_body_2_name(const std::string& value);
  std::string* _internal_mutable_body_2_name();
  public:

  // required .gazebo.msgs.Wrench body_1_wrench = 5;
  bool has_body_1_wrench() const;
  private:
  bool _internal_has_body_1_wrench() const;
  public:
  void clear_body_1_wrench();
  const ::gazebo::msgs::Wrench& body_1_wrench() const;
  ::gazebo::msgs::Wrench* release_body_1_wrench();
  ::gazebo::msgs::Wrench* mutable_body_1_wrench();
  void set_allocated_body_1_wrench(::gazebo::msgs::Wrench* body_1_wrench);
  private:
  const ::gazebo::msgs::Wrench& _internal_body_1_wrench() const;
  ::gazebo::msgs::Wrench* _internal_mutable_body_1_wrench();
  public:
  void unsafe_arena_set_allocated_body_1_wrench(
      ::gazebo::msgs::Wrench* body_1_wrench);
  ::gazebo::msgs::Wrench* unsafe_arena_release_body_1_wrench();

  // required .gazebo.msgs.Wrench body_2_wrench = 6;
  bool has_body_2_wrench() const;
  private:
  bool _internal_has_body_2_wrench() const;
  public:
  void clear_body_2_wrench();
  const ::gazebo::msgs::Wrench& body_2_wrench() const;
  ::gazebo::msgs::Wrench* release_body_2_wrench();
  ::gazebo::msgs::Wrench* mutable_body_2_wrench();
  void set_allocated_body_2_wrench(::gazebo::msgs::Wrench* body_2_wrench);
  private:
  const ::gazebo::msgs::Wrench& _internal_body_2_wrench() const;
  ::gazebo::msgs::Wrench* _internal_mutable_body_2_wrench();
  public:
  void unsafe_arena_set_allocated_body_2_wrench(
      ::gazebo::msgs::Wrench* body_2_wrench);
  ::gazebo::msgs::Wrench* unsafe_arena_release_body_2_wrench();

  // required uint32 body_1_id = 2;
  bool has_body_1_id() const;
  private:
  bool _internal_has_body_1_id() const;
  public:
  void clear_body_1_id();
  ::PROTOBUF_NAMESPACE_ID::uint32 body_1_id() const;
  void set_body_1_id(::PROTOBUF_NAMESPACE_ID::uint32 value);
  private:
  ::PROTOBUF_NAMESPACE_ID::uint32 _internal_body_1_id() const;
  void _internal_set_body_1_id(::PROTOBUF_NAMESPACE_ID::uint32 value);
  public:

  // required uint32 body_2_id = 4;
  bool has_body_2_id() const;
  private:
  bool _internal_has_body_2_id() const;
  public:
  void clear_body_2_id();
  ::PROTOBUF_NAMESPACE_ID::uint32 body_2_id() const;
  void set_body_2_id(::PROTOBUF_NAMESPACE_ID::uint32 value);
  private:
  ::PROTOBUF_NAMESPACE_ID::uint32 _internal_body_2_id() const;
  void _internal_set_body_2_id(::PROTOBUF_NAMESPACE_ID::uint32 value);
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.JointWrench)
 private:
  class _Internal;

  // helper for ByteSizeLong()
  size_t RequiredFieldsByteSizeFallback() const;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr body_1_name_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr body_2_name_;
  ::gazebo::msgs::Wrench* body_1_wrench_;
  ::gazebo::msgs::Wrench* body_2_wrench_;
  ::PROTOBUF_NAMESPACE_ID::uint32 body_1_id_;
  ::PROTOBUF_NAMESPACE_ID::uint32 body_2_id_;
  friend struct ::TableStruct_joint_5fwrench_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// JointWrench

// required string body_1_name = 1;
inline bool JointWrench::_internal_has_body_1_name() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool JointWrench::has_body_1_name() const {
  return _internal_has_body_1_name();
}
inline void JointWrench::clear_body_1_name() {
  body_1_name_.ClearToEmpty(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
  _has_bits_[0] &= ~0x00000001u;
}
inline const std::string& JointWrench::body_1_name() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointWrench.body_1_name)
  return _internal_body_1_name();
}
inline void JointWrench::set_body_1_name(const std::string& value) {
  _internal_set_body_1_name(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.JointWrench.body_1_name)
}
inline std::string* JointWrench::mutable_body_1_name() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.JointWrench.body_1_name)
  return _internal_mutable_body_1_name();
}
inline const std::string& JointWrench::_internal_body_1_name() const {
  return body_1_name_.Get();
}
inline void JointWrench::_internal_set_body_1_name(const std::string& value) {
  _has_bits_[0] |= 0x00000001u;
  body_1_name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), value, GetArena());
}
inline void JointWrench::set_body_1_name(std::string&& value) {
  _has_bits_[0] |= 0x00000001u;
  body_1_name_.Set(
    &::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::move(value), GetArena());
  // @@protoc_insertion_point(field_set_rvalue:gazebo.msgs.JointWrench.body_1_name)
}
inline void JointWrench::set_body_1_name(const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  _has_bits_[0] |= 0x00000001u;
  body_1_name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::string(value),
              GetArena());
  // @@protoc_insertion_point(field_set_char:gazebo.msgs.JointWrench.body_1_name)
}
inline void JointWrench::set_body_1_name(const char* value,
    size_t size) {
  _has_bits_[0] |= 0x00000001u;
  body_1_name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::string(
      reinterpret_cast<const char*>(value), size), GetArena());
  // @@protoc_insertion_point(field_set_pointer:gazebo.msgs.JointWrench.body_1_name)
}
inline std::string* JointWrench::_internal_mutable_body_1_name() {
  _has_bits_[0] |= 0x00000001u;
  return body_1_name_.Mutable(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
}
inline std::string* JointWrench::release_body_1_name() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.JointWrench.body_1_name)
  if (!_internal_has_body_1_name()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000001u;
  return body_1_name_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
}
inline void JointWrench::set_allocated_body_1_name(std::string* body_1_name) {
  if (body_1_name != nullptr) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  body_1_name_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), body_1_name,
      GetArena());
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.JointWrench.body_1_name)
}

// required uint32 body_1_id = 2;
inline bool JointWrench::_internal_has_body_1_id() const {
  bool value = (_has_bits_[0] & 0x00000010u) != 0;
  return value;
}
inline bool JointWrench::has_body_1_id() const {
  return _internal_has_body_1_id();
}
inline void JointWrench::clear_body_1_id() {
  body_1_id_ = 0u;
  _has_bits_[0] &= ~0x00000010u;
}
inline ::PROTOBUF_NAMESPACE_ID::uint32 JointWrench::_internal_body_1_id() const {
  return body_1_id_;
}
inline ::PROTOBUF_NAMESPACE_ID::uint32 JointWrench::body_1_id() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointWrench.body_1_id)
  return _internal_body_1_id();
}
inline void JointWrench::_internal_set_body_1_id(::PROTOBUF_NAMESPACE_ID::uint32 value) {
  _has_bits_[0] |= 0x00000010u;
  body_1_id_ = value;
}
inline void JointWrench::set_body_1_id(::PROTOBUF_NAMESPACE_ID::uint32 value) {
  _internal_set_body_1_id(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.JointWrench.body_1_id)
}

// required string body_2_name = 3;
inline bool JointWrench::_internal_has_body_2_name() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool JointWrench::has_body_2_name() const {
  return _internal_has_body_2_name();
}
inline void JointWrench::clear_body_2_name() {
  body_2_name_.ClearToEmpty(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
  _has_bits_[0] &= ~0x00000002u;
}
inline const std::string& JointWrench::body_2_name() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointWrench.body_2_name)
  return _internal_body_2_name();
}
inline void JointWrench::set_body_2_name(const std::string& value) {
  _internal_set_body_2_name(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.JointWrench.body_2_name)
}
inline std::string* JointWrench::mutable_body_2_name() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.JointWrench.body_2_name)
  return _internal_mutable_body_2_name();
}
inline const std::string& JointWrench::_internal_body_2_name() const {
  return body_2_name_.Get();
}
inline void JointWrench::_internal_set_body_2_name(const std::string& value) {
  _has_bits_[0] |= 0x00000002u;
  body_2_name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), value, GetArena());
}
inline void JointWrench::set_body_2_name(std::string&& value) {
  _has_bits_[0] |= 0x00000002u;
  body_2_name_.Set(
    &::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::move(value), GetArena());
  // @@protoc_insertion_point(field_set_rvalue:gazebo.msgs.JointWrench.body_2_name)
}
inline void JointWrench::set_body_2_name(const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  _has_bits_[0] |= 0x00000002u;
  body_2_name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::string(value),
              GetArena());
  // @@protoc_insertion_point(field_set_char:gazebo.msgs.JointWrench.body_2_name)
}
inline void JointWrench::set_body_2_name(const char* value,
    size_t size) {
  _has_bits_[0] |= 0x00000002u;
  body_2_name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::string(
      reinterpret_cast<const char*>(value), size), GetArena());
  // @@protoc_insertion_point(field_set_pointer:gazebo.msgs.JointWrench.body_2_name)
}
inline std::string* JointWrench::_internal_mutable_body_2_name() {
  _has_bits_[0] |= 0x00000002u;
  return body_2_name_.Mutable(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
}
inline std::string* JointWrench::release_body_2_name() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.JointWrench.body_2_name)
  if (!_internal_has_body_2_name()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000002u;
  return body_2_name_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
}
inline void JointWrench::set_allocated_body_2_name(std::string* body_2_name) {
  if (body_2_name != nullptr) {
    _has_bits_[0] |= 0x00000002u;
  } else {
    _has_bits_[0] &= ~0x00000002u;
  }
  body_2_name_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), body_2_name,
      GetArena());
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.JointWrench.body_2_name)
}

// required uint32 body_2_id = 4;
inline bool JointWrench::_internal_has_body_2_id() const {
  bool value = (_has_bits_[0] & 0x00000020u) != 0;
  return value;
}
inline bool JointWrench::has_body_2_id() const {
  return _internal_has_body_2_id();
}
inline void JointWrench::clear_body_2_id() {
  body_2_id_ = 0u;
  _has_bits_[0] &= ~0x00000020u;
}
inline ::PROTOBUF_NAMESPACE_ID::uint32 JointWrench::_internal_body_2_id() const {
  return body_2_id_;
}
inline ::PROTOBUF_NAMESPACE_ID::uint32 JointWrench::body_2_id() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointWrench.body_2_id)
  return _internal_body_2_id();
}
inline void JointWrench::_internal_set_body_2_id(::PROTOBUF_NAMESPACE_ID::uint32 value) {
  _has_bits_[0] |= 0x00000020u;
  body_2_id_ = value;
}
inline void JointWrench::set_body_2_id(::PROTOBUF_NAMESPACE_ID::uint32 value) {
  _internal_set_body_2_id(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.JointWrench.body_2_id)
}

// required .gazebo.msgs.Wrench body_1_wrench = 5;
inline bool JointWrench::_internal_has_body_1_wrench() const {
  bool value = (_has_bits_[0] & 0x00000004u) != 0;
  PROTOBUF_ASSUME(!value || body_1_wrench_ != nullptr);
  return value;
}
inline bool JointWrench::has_body_1_wrench() const {
  return _internal_has_body_1_wrench();
}
inline const ::gazebo::msgs::Wrench& JointWrench::_internal_body_1_wrench() const {
  const ::gazebo::msgs::Wrench* p = body_1_wrench_;
  return p != nullptr ? *p : *reinterpret_cast<const ::gazebo::msgs::Wrench*>(
      &::gazebo::msgs::_Wrench_default_instance_);
}
inline const ::gazebo::msgs::Wrench& JointWrench::body_1_wrench() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointWrench.body_1_wrench)
  return _internal_body_1_wrench();
}
inline void JointWrench::unsafe_arena_set_allocated_body_1_wrench(
    ::gazebo::msgs::Wrench* body_1_wrench) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(body_1_wrench_);
  }
  body_1_wrench_ = body_1_wrench;
  if (body_1_wrench) {
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.JointWrench.body_1_wrench)
}
inline ::gazebo::msgs::Wrench* JointWrench::release_body_1_wrench() {
  _has_bits_[0] &= ~0x00000004u;
  ::gazebo::msgs::Wrench* temp = body_1_wrench_;
  body_1_wrench_ = nullptr;
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::gazebo::msgs::Wrench* JointWrench::unsafe_arena_release_body_1_wrench() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.JointWrench.body_1_wrench)
  _has_bits_[0] &= ~0x00000004u;
  ::gazebo::msgs::Wrench* temp = body_1_wrench_;
  body_1_wrench_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Wrench* JointWrench::_internal_mutable_body_1_wrench() {
  _has_bits_[0] |= 0x00000004u;
  if (body_1_wrench_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Wrench>(GetArena());
    body_1_wrench_ = p;
  }
  return body_1_wrench_;
}
inline ::gazebo::msgs::Wrench* JointWrench::mutable_body_1_wrench() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.JointWrench.body_1_wrench)
  return _internal_mutable_body_1_wrench();
}
inline void JointWrench::set_allocated_body_1_wrench(::gazebo::msgs::Wrench* body_1_wrench) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(body_1_wrench_);
  }
  if (body_1_wrench) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(body_1_wrench)->GetArena();
    if (message_arena != submessage_arena) {
      body_1_wrench = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, body_1_wrench, submessage_arena);
    }
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  body_1_wrench_ = body_1_wrench;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.JointWrench.body_1_wrench)
}

// required .gazebo.msgs.Wrench body_2_wrench = 6;
inline bool JointWrench::_internal_has_body_2_wrench() const {
  bool value = (_has_bits_[0] & 0x00000008u) != 0;
  PROTOBUF_ASSUME(!value || body_2_wrench_ != nullptr);
  return value;
}
inline bool JointWrench::has_body_2_wrench() const {
  return _internal_has_body_2_wrench();
}
inline const ::gazebo::msgs::Wrench& JointWrench::_internal_body_2_wrench() const {
  const ::gazebo::msgs::Wrench* p = body_2_wrench_;
  return p != nullptr ? *p : *reinterpret_cast<const ::gazebo::msgs::Wrench*>(
      &::gazebo::msgs::_Wrench_default_instance_);
}
inline const ::gazebo::msgs::Wrench& JointWrench::body_2_wrench() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointWrench.body_2_wrench)
  return _internal_body_2_wrench();
}
inline void JointWrench::unsafe_arena_set_allocated_body_2_wrench(
    ::gazebo::msgs::Wrench* body_2_wrench) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(body_2_wrench_);
  }
  body_2_wrench_ = body_2_wrench;
  if (body_2_wrench) {
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.JointWrench.body_2_wrench)
}
inline ::gazebo::msgs::Wrench* JointWrench::release_body_2_wrench() {
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Wrench* temp = body_2_wrench_;
  body_2_wrench_ = nullptr;
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::gazebo::msgs::Wrench* JointWrench::unsafe_arena_release_body_2_wrench() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.JointWrench.body_2_wrench)
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Wrench* temp = body_2_wrench_;
  body_2_wrench_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Wrench* JointWrench::_internal_mutable_body_2_wrench() {
  _has_bits_[0] |= 0x00000008u;
  if (body_2_wrench_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Wrench>(GetArena());
    body_2_wrench_ = p;
  }
  return body_2_wrench_;
}
inline ::gazebo::msgs::Wrench* JointWrench::mutable_body_2_wrench() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.JointWrench.body_2_wrench)
  return _internal_mutable_body_2_wrench();
}
inline void JointWrench::set_allocated_body_2_wrench(::gazebo::msgs::Wrench* body_2_wrench) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(body_2_wrench_);
  }
  if (body_2_wrench) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(body_2_wrench)->GetArena();
    if (message_arena != submessage_arena) {
      body_2_wrench = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, body_2_wrench, submessage_arena);
    }
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  body_2_wrench_ = body_2_wrench;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.JointWrench.body_2_wrench)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

typedef boost::shared_ptr<gazebo::msgs::JointWrench> JointWrenchPtr;
// @@protoc_insertion_point(namespace_scope)

}  // namespace msgs
}  // namespace gazebo

typedef const boost::shared_ptr<gazebo::msgs::JointWrench const> ConstJointWrenchPtr;
// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_joint_5fwrench_2eproto
