// Automatically generated
#include "gazebo/sensors/AltimeterSensor.hh"
#include "gazebo/sensors/CameraSensor.hh"
#include "gazebo/sensors/ContactSensor.hh"
#include "gazebo/sensors/DepthCameraSensor.hh"
#include "gazebo/sensors/ForceTorqueSensor.hh"
#include "gazebo/sensors/GaussianNoiseModel.hh"
#include "gazebo/sensors/GpsSensor.hh"
#include "gazebo/sensors/GpuRaySensor.hh"
#include "gazebo/sensors/ImuSensor.hh"
#include "gazebo/sensors/LogicalCameraSensor.hh"
#include "gazebo/sensors/MagnetometerSensor.hh"
#include "gazebo/sensors/MultiCameraSensor.hh"
#include "gazebo/sensors/Noise.hh"
#include "gazebo/sensors/RaySensor.hh"
#include "gazebo/sensors/RFIDSensor.hh"
#include "gazebo/sensors/RFIDTag.hh"
#include "gazebo/sensors/SensorsIface.hh"
#include "gazebo/sensors/Sensor.hh"
#include "gazebo/sensors/SensorTypes.hh"
#include "gazebo/sensors/SensorFactory.hh"
#include "gazebo/sensors/SensorManager.hh"
#include "gazebo/sensors/SonarSensor.hh"
#include "gazebo/sensors/WideAngleCameraSensor.hh"
#include "gazebo/sensors/WirelessReceiver.hh"
#include "gazebo/sensors/WirelessTransceiver.hh"
#include "gazebo/sensors/WirelessTransmitter.hh"

