syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface LaserScanStamped
/// \brief Message for an laser scan with a time


import "time.proto";
import "laserscan.proto";

message LaserScanStamped
{
  // Time when the data was captured
  required Time time          = 1;
  required LaserScan scan     = 2;
}
