syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Light 
/// \brief Message for a light



import "pose.proto";
import "vector3d.proto";
import "color.proto";

message Light
{
  required string name                   = 1;
  enum LightType
  {
    POINT       = 1;
    SPOT        = 2;
    DIRECTIONAL = 3;
  }
  optional LightType type                = 2;

  optional Pose pose                     = 3;
  optional Color diffuse                 = 4;
  optional Color specular                = 5;
  optional float attenuation_constant    = 6;
  optional float attenuation_linear      = 7;
  optional float attenuation_quadratic   = 8;
  optional Vector3d direction               = 9;
  optional float range                   = 10;
  optional bool cast_shadows             = 11;
  optional float spot_inner_angle        = 12;
  optional float spot_outer_angle        = 13;
  optional float spot_falloff            = 14;
  optional uint32 id                     = 15;
}
