syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface WorldModify
/// \brief A message that allows for modifying (open, close) worlds


message WorldModify
{
  required string world_name = 1;
  optional bool remove = 2;
  optional bool create = 3;
  optional bool cloned = 4;
  optional string cloned_uri = 5;
}
