// Automatically generated
#include "gazebo/physics/Actor.hh"
#include "gazebo/physics/AdiabaticAtmosphere.hh"
#include "gazebo/physics/Atmosphere.hh"
#include "gazebo/physics/AtmosphereFactory.hh"
#include "gazebo/physics/BallJoint.hh"
#include "gazebo/physics/Base.hh"
#include "gazebo/physics/BoxShape.hh"
#include "gazebo/physics/Collision.hh"
#include "gazebo/physics/CollisionState.hh"
#include "gazebo/physics/Contact.hh"
#include "gazebo/physics/ContactManager.hh"
#include "gazebo/physics/CylinderShape.hh"
#include "gazebo/physics/Entity.hh"
#include "gazebo/physics/FixedJoint.hh"
#include "gazebo/physics/HeightmapShape.hh"
#include "gazebo/physics/Hinge2Joint.hh"
#include "gazebo/physics/HingeJoint.hh"
#include "gazebo/physics/GearboxJoint.hh"
#include "gazebo/physics/Inertial.hh"
#include "gazebo/physics/Gripper.hh"
#include "gazebo/physics/Joint.hh"
#include "gazebo/physics/JointController.hh"
#include "gazebo/physics/JointWrench.hh"
#include "gazebo/physics/JointState.hh"
#include "gazebo/physics/Light.hh"
#include "gazebo/physics/LightState.hh"
#include "gazebo/physics/Link.hh"
#include "gazebo/physics/LinkState.hh"
#include "gazebo/physics/MapShape.hh"
#include "gazebo/physics/MeshShape.hh"
#include "gazebo/physics/Model.hh"
#include "gazebo/physics/ModelState.hh"
#include "gazebo/physics/MultiRayShape.hh"
#include "gazebo/physics/PhysicsIface.hh"
#include "gazebo/physics/PhysicsEngine.hh"
#include "gazebo/physics/PhysicsFactory.hh"
#include "gazebo/physics/PhysicsTypes.hh"
#include "gazebo/physics/PlaneShape.hh"
#include "gazebo/physics/PolylineShape.hh"
#include "gazebo/physics/Population.hh"
#include "gazebo/physics/PresetManager.hh"
#include "gazebo/physics/RayShape.hh"
#include "gazebo/physics/Road.hh"
#include "gazebo/physics/Shape.hh"
#include "gazebo/physics/ScrewJoint.hh"
#include "gazebo/physics/SliderJoint.hh"
#include "gazebo/physics/SphereShape.hh"
#include "gazebo/physics/State.hh"
#include "gazebo/physics/SurfaceParams.hh"
#include "gazebo/physics/UniversalJoint.hh"
#include "gazebo/physics/UserCmdManager.hh"
#include "gazebo/physics/Wind.hh"
#include "gazebo/physics/World.hh"
#include "gazebo/physics/WorldState.hh"

